LANG_CODES = {
    'afrikaans': 'af',
    'albanian': 'sq',
    'amharic': 'am',
    'arabic': 'ar',
    'armenian': 'hy',
    'azerbaijani': 'az',
    'basque': 'eu',
    'belarusian': 'be',
    'bengali': 'bn',
    'bosnian': 'bs',
    'bulgarian': 'bg',
    'catalan': 'ca',
    'cebuano': 'ceb',
    'chichewa': 'ny',
    'chinese (simplified)': 'zh-cn',
    'chinese (traditional)': 'zh-tw',
    'corsican': 'co',
    'croatian': 'hr',
    'czech': 'cs',
    'danish': 'da',
    'dutch': 'nl',
    'english': 'en',
    'esperanto': 'eo',
    'estonian': 'et',
    'filipino': 'tl',
    'finnish': 'fi',
    'french': 'fr',
    'frisian': 'fy',
    'galician': 'gl',
    'georgian': 'ka',
    'german': 'de',
    'greek': 'el',
    'gujarati': 'gu',
    'haitian creole': 'ht',
    'hausa': 'ha',
    'hawaiian': 'haw',
    'hebrew': 'he',
    'hindi': 'hi',
    'hmong': 'hmn',
    'hungarian': 'hu',
    'icelandic': 'is',
    'igbo': 'ig',
    'indonesian': 'id',
    'irish': 'ga',
    'italian': 'it',
    'japanese': 'ja',
    'javanese': 'jw',
    'kannada': 'kn',
    'kazakh': 'kk',
    'khmer': 'km',
    'korean': 'ko',
    'kurdish (kurmanji)': 'ku',
    'kyrgyz': 'ky',
    'lao': 'lo',
    'latin': 'la',
    'latvian': 'lv',
    'lithuanian': 'lt',
    'luxembourgish': 'lb',
    'macedonian': 'mk',
    'malagasy': 'mg',
    'malay': 'ms',
    'malayalam': 'ml',
    'maltese': 'mt',
    'maori': 'mi',
    'marathi': 'mr',
    'mongolian': 'mn',
    'myanmar (burmese)': 'my',
    'nepali': 'ne',
    'norwegian': 'no',
    'odia': 'or',
    'pashto': 'ps',
    'persian': 'fa',
    'polish': 'pl',
    'portuguese': 'pt',
    'punjabi': 'pa',
    'romanian': 'ro',
    'russian': 'ru',
    'samoan': 'sm',
    'scots gaelic': 'gd',
    'serbian': 'sr',
    'sesotho': 'st',
    'shona': 'sn',
    'sindhi': 'sd',
    'sinhala': 'si',
    'slovak': 'sk',
    'slovenian': 'sl',
    'somali': 'so',
    'spanish': 'es',
    'sundanese': 'su',
    'swahili': 'sw',
    'swedish': 'sv',
    'tajik': 'tg',
    'tamil': 'ta',
    'telugu': 'te',
    'thai': 'th',
    'turkish': 'tr',
    'ukrainian': 'uk',
    'urdu': 'ur',
    'uyghur': 'ug',
    'uzbek': 'uz',
    'vietnamese': 'vi',
    'welsh': 'cy',
    'xhosa': 'xh',
    'yiddish': 'yi',
    'yoruba': 'yo',
    'zulu': 'zu'}

LANGUAGES = {value:key for key, value in LANG_CODES.items()}



import http.client
import hashlib
import urllib
import random
import json

import nltk
from nltk.tokenize import sent_tokenize
from BackTranslation.translated import Translated

try:
    nltk.data.find('tokenizers/punkt')
except LookupError:
    nltk.download('punkt')
class BackTranslation_Baidu(object):

    def __init__(self, appid, secretKey):
        if not appid:
            raise ValueError("'{}': INVALID appid, please register in http://api.fanyi.baidu.com/.".format(appid))

        if not secretKey:
            raise ValueError("'{}': INVALID secretKey, please register in http://api.fanyi.baidu.com/.".format(secretKey))

        self.appid = appid
        self.secretKey = secretKey
        self.MAX_LENGTH = 15
        self.httpClient = http.client.HTTPConnection('api.fanyi.baidu.com')
        self.queryURL = '/api/trans/vip/translate'




    def translate(self, text, src='auto', tmp=None):
        if src == 'auto':
            src = self._get_srcLang(self._sendRequest(text, src, 'en'))

        if not tmp: # if tmp is None, set the default tmp language
            if src == 'en':
                tmp = 'zh'
            else:
                tmp = 'en'

        if src == tmp:
            raise ValueError("Transited language ({tmp}) should different from srouce language ({src}).".format(tmp=tmp, src=src))

        # check the length of sentence
        if len(text) > self.MAX_LENGTH:
            original_sentences = self._split_segement(sent_tokenize(text))

            # translate the sentence one by one
            tran_text = []
            back_text = []
            for sentence in original_sentences:
                # language A --> language B
                mid = self._get_translatedText(self._sendRequest(sentence, src, tmp))
                tran_text.append(mid)

                # language B --> language A
                back_text.append(self._get_translatedText(self._sendRequest(sentence, tmp, src)))
            tran_text = ' '.join(tran_text)
            back_text = ' '.join(back_text)
        else:
            tran_text = self._get_translatedText(self._sendRequest(text, src, tmp))
            back_text = self._get_translatedText(self._sendRequest(text, tmp, src))
        result = Translated(src_lang=src, tmp_lang=tmp, text=text, trans_text=tran_text, back_text=back_text)
        return result


    def _sendRequest(self, text, src, tmp):
        salt = random.randint(32768, 65536)
        sign = hashlib.md5((self.appid + text + str(salt) + self.secretKey).encode()).hexdigest()
        url = self.queryURL + '?appid=' + self.appid + '&q=' + urllib.parse.quote(text) + '&from=' + src + '&to=' + tmp + '&salt=' + str(salt) + '&sign=' + sign

        try:
            self.httpClient.request('GET', url)
            response = self.httpClient.getresponse()
            result_all = response.read().decode("utf-8")
            result = json.loads(result_all)
        except Exception as e:
            print("'{}': Connection error.".format(e))

        return result

    def _split_segement(self, sentences):
        """
        Split the long sentences into multiple sentences whose lengths are less than MAX_LENGTH.

        :param sentences: the list of tokenized sentences from source text
        :return: the list of sentences with proper length
        :rtype: list
        """
        sentences_list = []
        block = ""
        for sentence in sentences:
            if len((block.rstrip() + ' ' + sentence).encode('utf-8')) > self.MAX_LENGTH:
                sentences_list.append(block.rstrip())
                block = sentence
            else:
                block = block + sentence + ' '
        sentences_list.append(block.rstrip())
        return sentences_list

    def closeHTTP(self):
        self.httpClient.close()

    def _get_srcLang(self, result):
        return result['from']

    def _get_translatedText(self, result):
        return result['trans_result'][0]['dst']


from googletrans import Translator
import nltk
from nltk.tokenize import sent_tokenize
import time
from BackTranslation.languages import LANGUAGES, LANG_CODES
from BackTranslation.translated import Translated

try:
    nltk.data.find('tokenizers/punkt')
except LookupError:
    nltk.download('punkt')



class BackTranslation(object):
    """
    Translate the words to other language and translate back
    in order to generate more samples for NLP.
    
    This package built based on googletrans package.
    Thus, you have to create only one instance to translate your words.
    Otherwise, an error will occur from googletrans.

    """
    def __init__(self, url=['translate.google.com']):
        self.translator = Translator(service_urls=url)
        self.Languages = LANGUAGES
        self.langCodes = LANG_CODES
        self.MAX_LENGTH = 5000


    def translate(self, text, src = None, tmp = None, sleeping = 0):

        if not src:
            src = self.translator.detect(text).lang

        if src.lower() not in self.Languages:
            raise ValueError("'{}': INVALID source language.".format(src))

        # if tmp is null, set a default language for tmp.
        if not tmp:
            if src == 'en':
                tmp = 'zh-cn'
            else:
                tmp = 'en'

        if tmp not in self.Languages:
            raise ValueError("'{}': INVALID transited language.".format(tmp))

        if src == tmp:
            raise ValueError("Transited language ({tmp}) should different from srouce language ({src}).".format(tmp=self.langCodes[tmp], src=self.langCodes[src]))

        # check the length of text
        if len(text) > self.MAX_LENGTH:
            original_sentences = self._split_segement(sent_tokenize(text))

            t_text = self.translator.translate(original_sentences, src=src, dest=tmp)
            tran_text = ' '.join([t.text for t in t_text])
            time.sleep(sleeping)
            r_text = self.translator.translate([t.text for t in t_text], src=tmp, dest=src)
            back_text = ' '.join([r.text for r in r_text])
            back_text.rstrip()
        else:
            mid_text = self.translator.translate(text, src=src, dest=tmp)
            tran_text = mid_text.text
            time.sleep(sleeping)  # Sleep between translation
            result_text = self.translator.translate(tran_text, src=tmp, dest=src)
            back_text = result_text.text
        result = Translated(src_lang=src, tmp_lang=tmp, text=text, trans_text=tran_text, back_text=back_text)
        return result





    def _split_segement(self, sentences):
        """
        Split the long sentences into multiple sentences whose lengths are less than MAX_LENGTH.

        :param sentences: the list of tokenized sentences from source text
        :return: the list of sentences with proper length
        :rtype: list
        """
        sentences_list = []
        block = ""
        for sentence in sentences:
            if len((block.rstrip() + ' ' + sentence).encode('utf-8')) > self.MAX_LENGTH:
                sentences_list.append(block.rstrip())
                block = sentence
            else:
                block = block + sentence + ' '
        sentences_list.append(block.rstrip())
        return sentences_list


    def searchLanguage(self, language):
        """
        Search a matched language code by language name.

        :param language: the full name of language in english
        :return: return matched language name and their code
        :rtype: dict

        Usage:
            >>> trans = BackTranslation()
            >>> trans.searchLanguage('frenc')
            # {'french': 'fr'}
            >>> trans.searchLanguage('Chinese')
            # {'chinese (simplified)': 'zh-cn', 'chinese (traditional)': 'zh-tw'}
        """
        language = language.lower()
        if language in self.langCodes:
            return {language: self.langCodes[language]}
        else:
            language_list = [key for key in self.langCodes.keys() if language in key]
            if language_list != []:
                return {lan: self.langCodes[lan] for lan in language_list}
            else:
                raise ValueError("{}: No existing language.".format(language))





class Translated(object):
    """
    Save the translated result
    """
    def __init__(self, src_lang, tmp_lang, text, trans_text, back_text):
        self.source_text = text
        self.src = src_lang
        self.tmp = tmp_lang
        self.tran_text = trans_text
        self.result_text = back_text


    def __str__(self):
        return self.__unicode__()



    def __unicode__(self):
        if len(self.result_text) > 30 :
            show_text = ' '.join(self.result_text.split(' ')[:30])
            show_text += '...'
        return u'Trabslated(src={src}, tmp={tmp}, result_text={text})'.format(src=self.src, tmp=self.tmp, text = self.result_text)


from distutils.core import setup
import setuptools

setup(
    name = 'BackTranslation',
    version = "0.2.2",
    author = "Zhiqiang Wu",
    author_email = "wzq0515@gmail.com",
    license = "MIT",
    url = "https://github.com/hhhwwwuuu/BackTranslation",
    description = "Back translation for Natural Language Processing (NLP) using Google Translate",
    long_description = open('README.md', 'r', encoding='utf-8').read(),
    long_description_content_type="text/markdown",

    python_requires='>=3',
    packages=setuptools.find_packages(),
    keywords = ['Translation', 'NLP', 'back-translation'],
    classifiers=(
        "Operating System :: OS Independent",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.0",
    ),
    install_requires = ['googletrans==4.0.0rc1', 'nltk']
)